@@ -582,14 +582,15 @@ def _handle_json_option(ctx, param, value):
 
 @workspace_cli.command('merge')
 @click.argument('METS_PATH')
-@click.option('--overwrite/--no-overwrite', is_flag=True, default=False, help="Overwrite in case of file name conflicts with data from METS_PATH")
+@click.option('--overwrite/--no-overwrite', is_flag=True, default=False, help="Overwrite on-disk file in case of file name conflicts with data from METS_PATH")
+@click.option('--force/--no-force', is_flag=True, default=False, help="Overwrite mets:file from --mets with mets:file from METS_PATH if IDs clash")
 @click.option('--copy-files/--no-copy-files', is_flag=True, help="Copy files as well", default=True, show_default=True)
 @click.option('--fileGrp-mapping', help="JSON object mapping src to dest fileGrp", callback=_handle_json_option)
 @click.option('--fileId-mapping', help="JSON object mapping src to dest file ID", callback=_handle_json_option)
 @click.option('--pageId-mapping', help="JSON object mapping src to dest page ID", callback=_handle_json_option)
 @mets_find_options
 @pass_workspace
-def merge(ctx, overwrite, copy_files, filegrp_mapping, fileid_mapping, pageid_mapping, file_grp, file_id, page_id, mimetype, mets_path):   # pylint: disable=redefined-builtin
+def merge(ctx, overwrite, force, copy_files, filegrp_mapping, fileid_mapping, pageid_mapping, file_grp, file_id, page_id, mimetype, mets_path):   # pylint: disable=redefined-builtin
     """
     Merges this workspace with the workspace that contains ``METS_PATH``
 
@@ -607,6 +608,7 @@ def merge(ctx, overwrite, copy_files, filegrp_mapping, fileid_mapping, pageid_ma
     other_workspace = Workspace(ctx.resolver, directory=str(mets_path.parent), mets_basename=str(mets_path.name))
     workspace.merge(
         other_workspace,
+        force=force,
         overwrite=overwrite,
         copy_files=copy_files,
         fileGrp_mapping=filegrp_mapping,
@@ -615,7 +617,7 @@ def merge(ctx, overwrite, copy_files, filegrp_mapping, fileid_mapping, pageid_ma
         file_grp=file_grp,
         file_id=file_id,
         page_id=page_id,
-        mimetype=mimetype,
+        mimetype=mimetype
     )
     workspace.save_mets()
 
